#include <iostream>

using namespace std;

// Uzupelnienie siagu 'str' znakami 'ch' z lewej strony tak, aby nowy ciag mial 'newLength' dlugosci
string PadLeft(string str, int cnt, char ch)
{
	string result = str;
	for(int i=0; i<cnt; i++)
		result = ch + result;
	return result;
}

// Usniecie wszystkich znakow 'ch' z lewj strony ciagu 'str'
string StripLeft(string str, char ch)
{
	string result = str;
	while(result.length()>1 && result[0]==ch)
		result.erase(0, 1);					// Uzycie metody standardowej erase() - usuniecie najbardziej skrajnego znaku po lewej
	return result;
}

// Zamiana cyfry zapisanej jako znak na jej wartosc liczbowa
int DigitToInt(char digit)
{
	if(digit>='0' && digit<='9')		// Cyfry dziesietnie
		return digit - '0';
	else if(digit>='A' && digit<='Z')	// Cyfry szenastkowe zapisane wielkimi literami
		return digit - 'A' + 10;
	else if(digit>='a' && digit<='z')	// Cyfry szenastkowe zapisane malymi literami
		return digit - 'a' + 10;
	else								// Znak niebedacy cyfra
		return 0;
}

char IntToDigit(int value)
{
	if(value>=0 && value<=9)
		return value + '0';
	else if(value>=10 && value <=36)
		return value - 10 + 'A';
	else
		return '0';
}

int AnyBaseToDec(string value, int base)
{
	bool minusResult = false;
	if(value.length()>0 and value[0]=='-')
	{
		minusResult = true;
		value.erase(0, 1);
	}
	int result = 0;
	for(int i=0; i<value.length(); i++)
		result = result * base + DigitToInt(value[i]);
	if(minusResult)
		result = -result;
	return result;
}

string DecToAnyBase(int value, int base)
{
	if(value==0)
		return "0";

	bool minusResult = false;
	if(value<0)
	{
		minusResult = true;
		value = -value;
	}
	string result = "";
	while(value>0)
	{
		result = IntToDigit(value % base) + result;
		value /= base;
	}
	if(minusResult)
		result = '-' + result;
	return result;
}

// Dodawanie dwoch liczb w systemie binarnym
string AddBin(string value1, string value2)
{
	int len1 = value1.length(), len2 = value2.length();
	// Dopasowanie dlugosci - dodanie '0' z przodu w razie potrzeby
	if(len1>len2)
		value2 = PadLeft(value2, len1-len2, '0');
	else if (len2>len1)
		value1 = PadLeft(value1, len2-len1, '0');
	// Dodawanie
	string result = "";
	int carry = 0;					// Wartosc przeniesienia
	for(int i=value1.length()-1; i>=0; i--)
	{
		int sum = DigitToInt(value1[i]) + DigitToInt(value2[i]) + carry;
		carry = sum / 2;
		sum = sum % 2;
		result = IntToDigit(sum) + result;
	}
	result = IntToDigit(carry) + result;
	return StripLeft(result, '0');			// Usuniecie zbednych zer po lewej stronie
}

// Odejmowanie dwoch liczb w systemi binarnym
string SubBin(string value1, string value2)
{
	int len1 = value1.length(), len2 = value2.length();
	// Dopasowanie dlugosci - dodanie '0' z przodu w razie potrzeby
	if(len1>len2)
		value2 = PadLeft(value2, len1-len2, '0');
	else if (len2>len1)
		value1 = PadLeft(value1, len2-len1, '0');
	bool minusResult = false;
	// Sprawdzenie czy moze druga wartosc jest wieksza od pierwszej
	// W takim wypadku wynik bedzie ujemny i musimy zamienic kolejnosc operandow
	// Ciagi sa tej samej dlugosci, mozemy je wiec porownac leksykalnie
	if(value2>value1)
	{
		string t = value1;
		value1 = value2;
		value2 = t;
		minusResult = true;
	}
	// Odejmowanie
	string result = "";
	int borrow = 0;								// Pozyczka
	for(int i=value1.length()-1; i>=0; i--)
	{
		int diff = DigitToInt(value1[i]) - DigitToInt(value2[i]) - borrow;
		if(diff<0)								// Nalezy pozyczyc
		{
			diff += 2;
			borrow = 1;
		}
		else
			borrow = 0;
		result = IntToDigit(diff) + result;
	}
	result = StripLeft(result, '0');			// Usuniecie zbednych zer po lewej stronie
	if(minusResult)
		result = '-' + result;
	return result;
}

// Mnoenie dwoch liczb w systemie binarnym
string MulBin(string value1, string value2)
{
	// Zamieniamy w razie potrzeby wartosci zmiennych
	// tak, aby zmienna value2 byla krotsza - optymalizacja szybkosci
	if(value1.length()<value2.length())
	{
		string t = value1;
		value1 = value2;
		value2 = t;
	}
	// Mnozenie
	string result = "0";
	for(int i=value2.length()-1; i>=0; i--)
	{
		if(value2[i]!='0')						// Nalezy doda
			result = AddBin(result, value1);
		value1 = value1 + "0";					// Przesuniecie o jedna cyfre
	}
	result = StripLeft(result, '0');			// Usuniecie zbednych zer po lewej stronie
	return result;
}

void TestAdd(string value1, string value2)
{
	string addResult = AddBin(value1, value2);
	int v1 = AnyBaseToDec(value1, 2);
	int v2 = AnyBaseToDec(value2, 2);
	int addv = AnyBaseToDec(addResult, 2);
	bool ok = (v1 + v2 == addv);
	cout << value1 << "+" << value2 << "=" << addResult << "\tKontrola: " <<
		v1 << "+" << v2 << "=" << addv << (!ok ? "\tBLAD!" : "") << endl;
}

void TestSub(string value1, string value2)
{
	string subResult = SubBin(value1, value2);
	int v1 = AnyBaseToDec(value1, 2);
	int v2 = AnyBaseToDec(value2, 2);
	int subv = AnyBaseToDec(subResult, 2);
	bool ok = (v1 - v2 == subv);
	cout << value1 << "-" << value2 << "=" << subResult << "\tKontrola: " <<
		v1 << "-" << v2 << "=" << subv << (!ok ? "\tBLAD!" : "") << endl;
}

void TestMul(string value1, string value2)
{
	string mulResult = MulBin(value1, value2);
	int v1 = AnyBaseToDec(value1, 2);
	int v2 = AnyBaseToDec(value2, 2);
	int mulv = AnyBaseToDec(mulResult, 2);
	bool ok = (v1 * v2 == mulv);
	cout << value1 << "*" << value2 << "=" << mulResult << "\tKontrola: " <<
		v1 << "*" << v2 << "=" << mulv << (!ok ? "\tBLAD!" : "") << endl;
}

int main()
{
	TestAdd("0", "0");
	TestAdd("0", "1");
	TestAdd("1", "0");
	TestAdd("1", "1");
	TestAdd("1010", "1100");
	TestAdd("1010", "0100");
	TestAdd("0100", "1010");
	TestAdd("1010", "100");
	TestAdd("100", "1010");
	TestAdd("1111", "1111");
	TestSub("1110", "1011");
	TestSub("0", "0");
	TestSub("0", "1");
	TestSub("1", "0");
	TestSub("1", "1");
	TestSub("11001", "0");
	TestSub("11001", "1");
	TestSub("1010", "1100");
	TestSub("1010", "0100");
	TestSub("0100", "1010");
	TestSub("1010", "100");
	TestSub("100", "1010");
	TestSub("1111", "1111");
	for(int i=0; i<16; i++)
		for(int j=0; j<16; j++)
			TestSub(DecToAnyBase(i, 2), DecToAnyBase(j, 2));
	for(int i=0; i<16; i++)
		for(int j=0; j<16; j++)
			TestMul(DecToAnyBase(i, 2), DecToAnyBase(j, 2));
}
